package eu.jonahbauer.tichu.common.model.combinations;

import eu.jonahbauer.tichu.common.model.Card;
import eu.jonahbauer.tichu.common.model.CombinationType;
import eu.jonahbauer.tichu.common.model.Stack;
import eu.jonahbauer.tichu.common.model.exceptions.InvalidCombinationException;
import lombok.EqualsAndHashCode;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Unmodifiable;

import java.util.List;
import java.util.Objects;

@EqualsAndHashCode
abstract sealed class AbstractCombination implements Combination permits Tuple, Sequence, FullHouse, Single {
    private final @NotNull CombinationType type;
    private final @Unmodifiable @NotNull List<@NotNull Card> cards;

    AbstractCombination(@NotNull CombinationType type, @NotNull List<@NotNull Card> cards) {
        this.type = Objects.requireNonNull(type, "type must not be null");
        this.cards = List.copyOf(Objects.requireNonNull(cards, "cards must not be null"));

        if (!this.type.check(this.cards)) {
            throw new InvalidCombinationException(type, cards);
        }
    }

    @Override
    public @NotNull CombinationType type() {
        return type;
    }

    @Override
    public @Unmodifiable @NotNull List<@NotNull Card> cards() {
        return cards;
    }

    @Override
    public final @NotNull Card card(int index) {
        return cards.get(index);
    }

    @Override
    public final int size() {
        return cards.size();
    }

    @Override
    public boolean isCompatibleWith(@NotNull Stack stack) {
        return stack.isEmpty() || stack.top().type() == type() && stack.top().size() == size();
    }

    @Override
    public boolean isCompatibleAndHigher(@NotNull Stack stack) {
        return isCompatibleWith(stack) && isHigherThan(stack);
    }

    @Override
    public String toString() {
        return type + cards.toString();
    }
}
