package eu.jonahbauer.tichu.common.model.combinations;

import eu.jonahbauer.tichu.common.model.Card;
import eu.jonahbauer.tichu.common.model.CombinationType;
import eu.jonahbauer.tichu.common.model.Stack;
import org.jetbrains.annotations.NotNull;

import java.util.List;
import java.util.Objects;

import static eu.jonahbauer.tichu.common.model.CombinationType.BOMB;
import static eu.jonahbauer.tichu.common.model.CombinationType.BOMB_SEQUENCE;

/**
 * A combination of type {@link CombinationType#BOMB}
 */
public final class BombTuple extends Tuple implements Bomb {
    public static BombTuple of(@NotNull List<@NotNull Card> cards) {
        return new BombTuple(cards);
    }

    public static BombTuple of(@NotNull Card card1, @NotNull Card card2, @NotNull Card card3, @NotNull Card card4) {
        return new BombTuple(List.of(
                Objects.requireNonNull(card1, "card1 must not be null"),
                Objects.requireNonNull(card2, "card2 must not be null"),
                Objects.requireNonNull(card3, "card3 must not be null"),
                Objects.requireNonNull(card4, "card4 must not be null")
        ));
    }

    BombTuple(@NotNull List<@NotNull Card> cards) {
        super(BOMB, cards);
    }

    @Override
    public boolean isCompatibleWith(@NotNull Stack stack) {
        return Bomb.super.isCompatibleWith(stack);
    }

    @Override
    public boolean isHigherThan(@NotNull Stack stack) {
        assert isCompatibleWith(stack);
        return stack.isEmpty()
               || stack.top().type() != BOMB && stack.top().type() != BOMB_SEQUENCE
               || stack.top().type() == BOMB && value() > ((BombTuple) stack.top()).value();
    }
}
