package eu.jonahbauer.tichu.common.model.combinations;

import eu.jonahbauer.tichu.common.model.Card;
import eu.jonahbauer.tichu.common.model.CombinationType;
import eu.jonahbauer.tichu.common.model.Stack;
import org.jetbrains.annotations.Contract;
import org.jetbrains.annotations.NotNull;

import java.util.List;
import java.util.Objects;

/**
 * A combination of type {@link CombinationType#FULL_HOUSE}
 */
public final class FullHouse extends AbstractCombination {
    @Contract("_ -> new")
    public static @NotNull FullHouse of(@NotNull List<@NotNull Card> cards) {
        return new FullHouse(cards);
    }

    @Contract("_, _, _, _, _ -> new")
    public static @NotNull FullHouse of(
            @NotNull Card triple1, @NotNull Card triple2, @NotNull Card triple3,
            @NotNull Card pair1, @NotNull Card pair2
    ) {
        return new FullHouse(List.of(
                Objects.requireNonNull(triple1, "triple1 must not be null"),
                Objects.requireNonNull(triple2, "triple2 must not be null"),
                Objects.requireNonNull(triple3, "triple3 must not be null"),
                Objects.requireNonNull(pair1, "pair1 must not be null"),
                Objects.requireNonNull(pair2, "pair2 must not be null")
        ));
    }

    private final int triple;
    private final int pair;

    FullHouse(@NotNull List<@NotNull Card> cards) {
        super(CombinationType.FULL_HOUSE, cards);
        this.triple = Tuple.getSameValue(cards.subList(0, 3));
        this.pair = Tuple.getSameValue(cards.subList(3, 5));
    }

    /**
     * The {@linkplain Card#value() value} of the triple.
     */
    public int triple() {
        return triple;
    }

    /**
     * The {@linkplain Card#value() value} of the pair.
     */
    public int pair() {
        return pair;
    }

    @Override
    public boolean isHigherThan(@NotNull Stack stack) {
        assert isCompatibleWith(stack);
        return stack.isEmpty() || triple() > ((FullHouse) stack.top()).triple();
    }
}
