package eu.jonahbauer.tichu.common.model.combinations;

import eu.jonahbauer.tichu.common.model.Card;
import eu.jonahbauer.tichu.common.model.CombinationType;
import eu.jonahbauer.tichu.common.model.Stack;
import org.jetbrains.annotations.Contract;
import org.jetbrains.annotations.NotNull;

import java.util.List;
import java.util.Objects;

/**
 * A combination of type {@link CombinationType#SINGLE}
 */
public final class Single extends AbstractCombination {
    @Contract("_ -> new")
    public static @NotNull Single of(@NotNull List<@NotNull Card> cards) {
        return new Single(cards);
    }

    @Contract("_ -> new")
    public static @NotNull Single of(@NotNull Card card) {
        return new Single(List.of(Objects.requireNonNull(card, "card must not be null")));
    }

    Single(@NotNull List<@NotNull Card> cards) {
        super(CombinationType.SINGLE, cards);
    }

    /**
     * Returns the single card this combination consists of. This is shorthand for {@link #card(int) card(0)}.
     * @return the single card this combination consists of.
     */
    public @NotNull Card card() {
        return card(0);
    }

    /**
     * Returns the {@linkplain Card#value() value} of the {@linkplain #card() card}.
     * @return the value of the card.
     */
    public Integer value() {
        return card().value();
    }

    @Override
    public boolean isHigherThan(@NotNull Stack stack) {
        assert isCompatibleWith(stack);
        if (stack.isEmpty()) return true; // you can play any card onto an empty stack

        var top = (Single) stack.top();
        if (top.card() == Card.HOUND) return false; // you cannot play any card onto a hound

        return switch (card()) {
            case HOUND, MAHJONG -> false;
            case DRAGON -> true;
            case PHOENIX -> top.card() != Card.DRAGON;
            default -> {
                if (top.card() != Card.PHOENIX) {
                    yield value() > top.value();
                } else if (stack.size() == 1) {
                    // every normal card has a value of at least 2, which is higher that the phoenix 1½
                    yield true;
                } else {
                    yield value() > ((Single) stack.top(-1)).value();
                }
            }
        };
    }
}
