package eu.jonahbauer.tichu.common.model.combinations;

import eu.jonahbauer.tichu.common.model.Card;
import eu.jonahbauer.tichu.common.model.CombinationType;
import eu.jonahbauer.tichu.common.model.Stack;
import org.jetbrains.annotations.NotNull;

import java.util.List;
import java.util.NoSuchElementException;

public abstract sealed class Tuple extends AbstractCombination permits Pair, Triple, BombTuple {
    private final int value;

    Tuple(@NotNull CombinationType type, @NotNull List<@NotNull Card> cards) {
        super(type, cards);
        this.value = getSameValue(cards);
    }

    /**
     * The {@linkplain Card#value() value} of the cards in this tuple.
     */
    public int value() {
        return value;
    }

    @Override
    public boolean isHigherThan(@NotNull Stack stack) {
        assert isCompatibleWith(stack);
        return stack.isEmpty() || value() > ((Tuple) stack.top()).value();
    }

    /**
     * Assuming that all the {@linkplain Card#isNormal() normal cards} at the specified indices have the same value
     * returns that value. The return value is unspecified if the assumption is violated.
     * @param cards a list of cards
     * @throws NoSuchElementException if there is no normal card among the specified indices
     */
    static int getSameValue(@NotNull List<@NotNull Card> cards) {
        for (var card : cards) {
            if (card.isSpecial()) continue;

            return card.value();
        }
        throw new NoSuchElementException();
    }
}
